package rss;

import java.io.InputStream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.methods.GetMethod;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

/**
 * Responsible for reading RSS feeds from URLs.
 *
 * @author    Simon Brown
 */
public class RssReader {

  /**
   * Reads the RSS feed at the specified URL and returns an RssFeed instance
   * representing it.
   *
   * @param url   the URL of the RSS feed as a String
   * @return  an RssFeed instance representing the feed
   */
  public RssFeed read(String url) {
    RssFeed rssFeed = new RssFeed();

    try {
      HttpClient httpClient = new HttpClient();
      GetMethod getMethod = new GetMethod(url);
      int responseCode = httpClient.executeMethod(getMethod);

      if (responseCode != 200) {
        return rssFeed;
      }

      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      factory.setValidating(false);
      factory.setNamespaceAware(true);
      factory.setIgnoringElementContentWhitespace(true);
      factory.setIgnoringComments(true);
      DocumentBuilder builder = factory.newDocumentBuilder();
      builder.setErrorHandler(new ErrorHandler() {
        public void warning(SAXParseException e) throws SAXException {
          System.out.println(e);
          throw e;
        }

        public void error(SAXParseException e) throws SAXException {
          System.out.println(e);
          throw e;
        }

        public void fatalError(SAXParseException e) throws SAXException {
          System.out.println(e);
          throw e;
        }
      });

      InputStream in = getMethod.getResponseBodyAsStream();
      Document doc = builder.parse(in);
      Node root = doc.getDocumentElement();

      // this section supports RSS
      NodeList channels = doc.getElementsByTagName("channel");
      for (int i = 0; i < channels.getLength(); i++) {
        NodeList nodes = channels.item(i).getChildNodes();
        for (int j = 0; j < nodes.getLength(); j++) {
          Node n = nodes.item(j);

          if (n.getNodeName().equals("item")) {
            RssItem rssItem = loadRssItem(n);
            rssFeed.addItem(rssItem);
          }
        }
      }

      // this section supports RDF (a variation of RSS)
      // ideally RSS and RDF parsing would be separated, but this will
      // suffice for this simple example :-)
      NodeList items = doc.getElementsByTagName("item");
      for (int i = 0; i < items.getLength(); i++) {
        RssItem rssItem = loadRssItem(items.item(i));
        rssFeed.addItem(rssItem);
      }
    } catch (Exception e) {
      e.printStackTrace();
    }

    return rssFeed;
  }

  /**
   * Helper method to load an RSS item.
   *
   * @param root          the root Node describing the item
   * @throws Exception    if the item can't be loaded
   */
  private RssItem loadRssItem(Node root) throws Exception {
    String title = null;
    String link = null;

    NodeList nodes = root.getChildNodes();
    for (int i = 0; i < nodes.getLength(); i++) {
      Node n = nodes.item(i);

      if (n.getNodeName().equals("title")) {
        title = getTextValue(n);
      }

      if (n.getNodeName().equals("link")) {
        link = getTextValue(n);
      }
    }

    RssItem item = new RssItem();
    item.setTitle(title);
    item.setLink(link);
    return item;
  }

  /**
   * Helper method to extract the text value from a given node.
   *
   * @param node    a Node
   * @return    the text value, or an empty string if no text value available
   */
  private String getTextValue(Node node) {
    if (node.hasChildNodes()) {
      return node.getFirstChild().getNodeValue();
    } else {
      return "";
    }
  }

}
